﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;

namespace PB069.Tutorial13.WinFormsCounter
{
    /*
     * Program vypisujici pomoci tri vlaken cisla od 1 do 50 do seznamu.
     * 
     * Pomoci bloku lock synchronizujeme vlakna tak, aby 
     * se vypsala vsechna cisla a ve spravnem poradi.
     */

    public partial class MainForm : Form
    {
        private int number = 0;
        private object numberIncrementLock = new object();

        public MainForm()
        {
            InitializeComponent();
        }

        private void runThreadsButton_Click(object sender, EventArgs e)
        {
            number = 0;
            numbersListBox.Items.Clear();

            Thread thread1 = new Thread(IncrementNumber);
            Thread thread2 = new Thread(IncrementNumber);
            Thread thread3 = new Thread(IncrementNumber);

            thread1.Start();
            thread2.Start();
            thread3.Start();
        }

        private void IncrementNumber()
        {
            Action<int> addNumberToListBoxDelegate = AddNumberToListBox;

            while (true)
            {
                /*
                 * Zamknutim nasledujici casti kodu zajistime, ze si vlakna nebudou 
                 * menit hodnotu polozky number 'pod rukama'.
                 */

                lock (numberIncrementLock)
                {
                    if (number >= 50)
                    {
                        break;
                    }
                    number++;

                    /*
                     * K ovladacim prvkum lze pristoupit jen z vlakna, ktere je 
                     * vytvorilo (coz je typicky primarni vlakno procesu).
                     * 
                     * Jestlize potrebujete pracovat s ovladacimi prvky z vlakna,
                     * ktere jej nevlastni, muzete o tom pozadat pomoci metody Invoke.
                     * 
                     * Metode Invoke ze tridy Control se predava delegat zastupujici metodu, 
                     * kterou chcete vyvolat, kterou nasleduje seznam parametru metody 
                     * ve spravnem poradi.
                     */

                    Invoke(addNumberToListBoxDelegate, number);
                }
            }
        }

        private void AddNumberToListBox(int number)
        {
            numbersListBox.Items.Add(number);
        }
    }
}
